/* 
* A primeira seção da especificação vai até o primeiro %%,
* e consiste de código Java que é copiado ao pé da letra
*
*/

package mhtml;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.io.IOException;

%%

/*
* A segunda seção vai até o próximo %%, e são diversos parâmetros
* de configuração, além de código Java copiado para o corpo da
* classe do analisador léxico
*
*/

%class Scanner          // nome da classe do analisador
%public                 // classe deve ser pública
%line                   // guarde número da linha em yyline
%column                 // guarde número da coluna em yycolumn
%function nextToken     // nome do método que vai fornecer um token
%type Token             // classe usado para tokens

// Código Java entre %{ e %} é copiado pro corpo da classe
// do analisador
%{ 

	public Scanner() { }

	public void input(String input) {
	    // inicializa entrada pro analisador
		yyreset(new StringReader(input));
	}
	
	public List<Token> allTokens() throws IOException {
		List<Token> tokens = new ArrayList<Token>();
		Token tok = nextToken();
		while(tok.type != Token.EOF) {
			tokens.add(tok);
			tok = nextToken();
		}
		return tokens;
	}

%}

// Scanners auxiliares, para tags, scripts e comentários
%state TAG, SCRIPT, COMMENT

%%

/*
* A última seção contém as regras léxicas, cada regra é um
* par com uma expressão regular e um trecho de código Java
* entre { e }.
*
*/

// Espaços são ignorados em todos os scanners
[ \r\n\t]    { }

/*
* Scanner de MHTML
*/
<YYINITIAL> {
  [<][!][-][-]    { yybegin(COMMENT); }
  [<]             { yybegin(TAG); return new Token('<', yytext(), yyline, yycolumn); }
  [<][/]          { yybegin(TAG); return new Token(Token.LS, yytext(), yyline, yycolumn); }
  [<][%]          { yybegin(SCRIPT); return new Token(Token.LP, yytext(), yyline, yycolumn); }
  [<][%][=]       { yybegin(SCRIPT); return new Token(Token.LPE, yytext(), yyline, yycolumn); }
}

/*
* Comentários MHTML
*/
<COMMENT> {
  [-][-][>]       { yybegin(YYINITIAL); }
  .               { }
}

/*
* Scanner de tags
*/
<TAG> {
  [>]                   { yybegin(YYINITIAL); return new Token('>', yytext(), yyline, yycolumn); }
  [/][>]                { yybegin(YYINITIAL); return new Token(Token.SG, yytext(), yyline, yycolumn); }
}

/*
* Scanner de scripts
*/
<SCRIPT> {
  [%][>]                 { yybegin(YYINITIAL); return new Token(Token.PG, yytext(), yyline, yycolumn); }
  function               { return new Token(Token.FUNCTION, yytext(), yyline, yycolumn); }
  end                    { return new Token(Token.END, yytext(), yyline, yycolumn); }
  while                  { return new Token(Token.WHILE, yytext(), yyline, yycolumn); }
  local                  { return new Token(Token.LOCAL, yytext(), yyline, yycolumn); }
  true                   { return new Token(Token.TRUE, yytext(), yyline, yycolumn); }
  false                  { return new Token(Token.FALSE, yytext(), yyline, yycolumn); }
  and                    { return new Token(Token.AND, yytext(), yyline, yycolumn); }
  else                   { return new Token(Token.ELSE, yytext(), yyline, yycolumn); }
  if                     { return new Token(Token.IF, yytext(), yyline, yycolumn); }
  elseif                 { return new Token(Token.ELSEIF, yytext(), yyline, yycolumn); }
  not                    { return new Token(Token.NOT, yytext(), yyline, yycolumn); }
  nil                    { return new Token(Token.NIL, yytext(), yyline, yycolumn); }
  or                     { return new Token(Token.OR, yytext(), yyline, yycolumn); }
  return                 { return new Token(Token.RETURN, yytext(), yyline, yycolumn); }
  then                   { return new Token(Token.THEN, yytext(), yyline, yycolumn); }
  [/]                    { return new Token('/', yytext(), yyline, yycolumn); }
  [<]                    { return new Token('<', yytext(), yyline, yycolumn); }
  [=]                    { return new Token('=', yytext(), yyline, yycolumn); }
  [+]                    { return new Token('+', yytext(), yyline, yycolumn); }
  [-]                    { return new Token('-', yytext(), yyline, yycolumn); }
  [*]                    { return new Token('*', yytext(), yyline, yycolumn); }
  [(]                    { return new Token('(', yytext(), yyline, yycolumn); }
  [)]                    { return new Token(')', yytext(), yyline, yycolumn); }
  [{]                    { return new Token('{', yytext(), yyline, yycolumn); }
  [}]                    { return new Token('}', yytext(), yyline, yycolumn); }
  [.]                    { return new Token('.', yytext(), yyline, yycolumn); }
  [,]                    { return new Token(',', yytext(), yyline, yycolumn); }
  ==                     { return new Token(Token.EQ, yytext(), yyline, yycolumn); }
  [~][=]                 { return new Token(Token.NEQ, yytext(), yyline, yycolumn); }
  [.][.]                 { return new Token(Token.CONCAT, yytext(), yyline, yycolumn); }
}

// A regra de EOF é a mesma para todos os scanners
<<EOF>>      { return new Token(Token.EOF, "", yyline, yycolumn); }

// Erros léxicos são tratados da mesma forma para todos os scanners
.            { throw new RuntimeException("erro léxico, linha: " + 
               (yyline+1) + ", coluna : " + (yycolumn+1) + ", char: " + 
               yytext()); }

